################################################################################

# Tatvic Blog : Google analytics data extraction in R
#        link : http://www.tatvic.com/blog/ga-data-extraction-in-r/

# Instructions :
#               This file contains R codes to extract Google Analytics data 
#               Before start using this codes you need to install R packages like 
#               RCurl and rjon. This code will work in Linux as well as Windows os.


# Loading the R libraries RCurl, rjson 
library(RCurl)
library(rjson)

# For windows user for specifying subdirectory for cacert.pem
options(RCurlOptions = list(capath = system.file("CurlSSL", "cacert.pem", package = "RCurl"), ssl.verifypeer = FALSE))

# Defining GA data by metrics and dimension

# Set the dimensions and metrics for Google analytics search query
ga_dimensions <- 'ga:visitorType,ga:operatingSystem,ga:country'
ga_matrics <- 'ga:visits,ga:bounces,ga:avgTimeOnSite'

# set the starting and ending date 
startdate <- '2012-01-01'
enddate <- '2012-11-30'

# Set the segment, sort and filters   
segment <- 'dynamic::ga:operatingSystem==Android'
sort <- 'ga:visits'
filters <- 'ga:country==Canada' 

# Set the access token from Oauth 2.0 authorizing with your GA account 
access_token <- 'your access token'

# Initiate the dataframe to store the output
finalres <- data.frame()


# For requesting the GA profiles and store the JSON response in to GA.profiles.Json variable
GA.profiles.Json <- getURL(paste("https://www.googleapis.com/analytics/v3/management/accounts/~all/webproperties/~all/profiles?access_token=",access_token,sep="", collapse = ","))

# To convert resonse variable GA.profiles.Json to list
GA.profiles.List <- fromJSON(GA.profiles.Json, method='C')

# Storing the profile id and name to profile.id and profile.name variable

GA.profiles.param <- t(sapply(GA.profiles.List$items,
                              '[', 1:max(sapply(GA.profiles.List$items, length)))) 
profiles.id <- as.character(GA.profiles.param[, 1])
profiles.name <- as.character(GA.profiles.param[, 7])


# Storing the profile.id and profile.name to profiles data.frame
profiles <- data.frame(id=profiles.id,name=profiles.name)

# To list down profiles
profiles

# Set your google analytics profile ID here
profileid <- 'ga:10696290'


# Start_index variable for managing the starting index for requesting URI
start_index <- 0

repeat 
{
  
  GA.Data<-getURL(paste('https://www.googleapis.com/analytics/v3/data/ga?',
                        'ids=',profileid,
                        '&start-date=',startdate,
                        '&end-date=',enddate,
                        '&metrics=',ga_matrics,
                        '&dimensions=',ga_dimensions,
                        '&max-results=',10000,
                        '&start-index=',start_index*10000+1,
                        '&sort=',sort,
                        '&filters=',filters,
                        '&segment=',segment,
                        '&access_token=',access_token, sep='', collapse=''))  
  

  # For converting the Json data to list object GA.list
  GA.list <- fromJSON(GA.Data, method='C')
	
  # For getting the total numbers of the data rows  
  totalrow <-  GA.list$totalResults 
  
  # For calculating the total extracted rows  
  totalrows <- nrow(do.call(rbind, as.list(GA.list$rows))) + start_index*10000
  finalres <- rbind(finalres,do.call(rbind, as.list(GA.list$rows)))
  start_index <- start_index + 1
  
  # For checking whether the total extracted data is reached at limit 
  #  where totalrow is to be extracted total rows 
  # and totalrows is total numbers of the extracted data rows

  
  if(totalrow==totalrows)
  {
    
    #splitting the ga_matrics to vectors 
    metrics_vec <- unlist(strsplit(ga_matrics,split=','))
    
    # Splitting the ga_dimensions to vectors 
    dimension_vec <-unlist(strsplit(ga_dimensions,split=','))
    
    # To splitting the columns name from string object(dimension_vec)
    ColnamesDimension <- gsub('ga:','',dimension_vec)

    # To splitting the columns name from string object(metrics_vec)
    ColnamesMetric <- gsub('ga:','',metrics_vec)
   
    # Combining dimension and metric column anmes to col_names
    col_names <- c(ColnamesDimension,ColnamesMetric)
    colnames(finalres) <- col_names
    
    # To convert the object GArows to dataframe type
    GA.DF <- as.data.frame(finalres)
    
    print(paste("Your query matched",
                nrow(GA.DF),"results that are stored to dataframe GA.DF"))
    
    break
  }
}

# To check the top rows of the extracted dataset
head(GA.DF)
